/*=============================================================
  Linux News C言語版 Copyright (C)2018 etokami
  Ver 0.0001α(2018/01/01)
  =============================================================*/

#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <time.h>
#include <unistd.h>


//辞書ファイルパスと一時ファイルパス
#define DIC_FILEPATH "./news.dic"
#define TMP_FILEPATH "./temp.tmp"


//一度に読み出すサイズと読み出せる制限
//COLUMN_SIZE分を読み込んでCOLUMN_LIMIT以上だとエラーになります
#define COLUMN_SIZE    2560		//１行読み出しバッファ
#define COLUMN_LIMIT   2048		//１行読み出しサイズ制限


//辞書ファイルの制限サイズ
#define CATEGORY_LIMIT 9999		//１カテゴリ内の容量制限
#define TEXT_LIMIT     2048		//実はCOLUMN_LIMIT依存
#define WORD_LIMIT      512		//単語の最大サイズ


//バッファメモリ関連
//お手軽感を出すために敢えてヒープは使っていません
//あまり巨大にし過ぎないようにすること
#define BUFFER_SIZE    8192		//本文変換用バッファ
#define TEXT_BUFFER    2048		//本文用バッファ
#define WORD_BUFFER    1024		//単語用バッファ


//以下変更禁止
#define COLOR_RED      "\033[91m"
#define COLOR_DEFAULT  "\033[39m"

#define SOFTWARE_FILE  "linux_news"
#define VERSION_INFO   "0.0001α"
#define COPYRIGHT      "Copyright (C)2018 etokami"


typedef   unsigned char   Uchar;
typedef   unsigned short  Ushort;
typedef   unsigned int    Uint;
typedef   unsigned long   Ulong;


//システム周りの構造体
typedef struct {

char help;			//ヘルプ表示フラグ
char make_dic;			//辞書ファイル作成フラグ
char rebuild_dic;		//辞書ファイル再構成フラグ
int  page_number;		//ページ番号選択用
char hide_ukagaka;		//AA非表示フラグ
char kakushi;			//隠しヘルプ表示フラグ
int  ukagaka;			//AA選択用

int  counter[7];		//カウンタ値保存配列
char base_color;		//ベースカラー用変数
char color_code[9];		//カラーコード用配列

} system_t;


//グローバル変数
char file_buffer[COLUMN_SIZE]={0};	//共通のファイルバッファ
int  errline=0;				//エラー行表示用


//プロトタイプ宣言
int  cc_fgets(char *,FILE *);
int  cc_strtol(int *,const char *);
int  calc_splitstr(int *,const char *,const char *);
int  split_token(char *,const int,const char *,char **);
void remove_char(char *,char);
int  get_config(int **,int,char **);
void get_colorcode(char *,const int,const int);
int  fileseek(const char *,FILE *);
void convert_text(char *,const char *,FILE *,const system_t *);
void rebuild_dictionary(void);
void make_dictionary(void);
void show_ukagaka(system_t *);
void show_error(int);
void help(char);
void set_option(int,char **,system_t *);



//
//  メイン処理
//
int main(int argc,char *argv[]) {

system_t system={0};

set_option(argc,argv,&system);

if (system.help) { help(system.kakushi); }

if (system.rebuild_dic) { rebuild_dictionary(); }
if (system.make_dic) { make_dictionary(); }


static char buf[BUFFER_SIZE]={0};
int  err;

FILE *fp;
fp=fopen(DIC_FILEPATH,"r");
if (fp == NULL) { show_error(10); }


//カラースキーム取得
if (fileseek("[color]",fp)) { show_error(16); } //エラー
if (cc_fgets(buf,fp)) { show_error(26); } //エラー

int   nbf[10];
int  *pn=nbf;
char *ps=buf;
if (get_config(&pn,9,&ps)) { show_error(45); }

for (int i=0 ; i<9 ; i++) {
      if ((nbf[i] < 0) || (nbf[i] > 9)) { show_error(45); } //エラー
      if (i == 0) { system.base_color=nbf[0]; }
             else { system.color_code[i-1]=(char)nbf[i]; }
}


//カウンタ値取得
if (fileseek("[counter]",fp)) { show_error(17); } //エラー
if (cc_fgets(buf,fp)) { show_error(27); } //エラー

pn=nbf;
ps=buf;
if (get_config(&pn,7,&ps)) { show_error(46); }

for (int i=0 ; i<7 ; i++) {
      if (nbf[i] > CATEGORY_LIMIT) { show_error(36+i); } //エラー
      system.counter[i]=nbf[i];
}


//乱数シードをセット、getpidは悪あがきの小細工
srand((unsigned)(time(NULL)+getpid()));

//本文取得
if (fileseek("[text]",fp)) { show_error(18); } //エラー
char text_str[TEXT_BUFFER];

int rnd=0;

//ページ指定されている場合は該当ページを選択
if ((system.page_number)
 && (system.counter[0] >= system.page_number))
      { rnd=system.page_number; }

if (rnd == 0) { rnd=rand()%system.counter[0]+1; }


//本文読み出し
for (int i=0 ; i<rnd ; i++) {
      if (err=cc_fgets(text_str,fp)) { if (err == 1) { show_error(28); }
                                                else { show_error(11); } }
      if (*text_str == '\0') { show_error(50); } //エラー
}


//単語を挿入
convert_text(buf,text_str,fp,&system);


//表示
show_ukagaka(&system);
printf("%s\n\n",buf);


fclose(fp);

return 0;


}



//  cc_fgets( d0 , s0 )
//  fgetsのついでに文字列の長さのチェックと
//  改行コード除去を一括で行う手抜き用関数
//  一度に読み出されるサイズはCOLUMN_SIZEとCOLUMN_LIMITで求めます
//
//  引数: d0 格納するchar型配列ポインタ
//        s0 ファイルポインタ
//
//  戻り値 --  0 正常終了
//         -- -1 異常終了(読み込まれた文字列が長すぎる)
//         --  1 異常終了(読み込みに失敗)
//         d0 -- 読み込まれた１行分のテキスト
//
int cc_fgets(char *buf,FILE *fp) {

if (fgets(buf,COLUMN_SIZE,fp) == NULL) { return 1; }

errline++;

if (strlen(buf) > COLUMN_LIMIT) { return -1; }

int pos=strlen(buf)-1;
if (pos >= 0) { if (buf[pos] == 0x0a) { buf[pos--]='\0'; } }
if (pos >= 0) { if (buf[pos] == 0x0d) { buf[pos--]='\0'; } }
if (pos >= 0) { if (buf[pos] == 0x0a) { buf[pos--]='\0'; } }

remove_char(buf,' ');

return 0;


}



//  cc_strtol( d0 , s0 )
//  strtolの後処理を一括で行う手抜き用関数
//  変換結果はint型にキャストされます
//
//  引数: d0 変換結果を格納するint型変数のポインタ
//        s0 変換元文字列ポインタ
//
//  戻り値: --  0 正常終了
//          -- -1 異常終了
//          d0 -- 変換結果がint型で格納されます
//
int cc_strtol(int *num,const char *str) {

char *end;
long nbf=strtol(str,&end,0);

if (*end != '\0') { return -1; }
if ( str ==  end) { return -1; }
if ((nbf < INT_MIN) || (nbf > INT_MAX)) { return -1; }

*num=(int)nbf;

return 0;


}



//  remove_char( d0 , s0 )
//  文字列中に含まれる指定１文字を削除する
//  本来は文字列中のスペース、タブ、コメントを消す機能です
//  通常使用ではs0を必ず半角スペースに指定してください
//
//  引数: d0 除去するchar型文字列ポインタ
//        s0 除去したいchar型文字変数
//
//  戻り値: d0 -- 直接変更が加えられます
//
//  s0に半角スペースを指定すると//で始まるコメントも削除されます
//  その後タブ削除のため一回だけ再帰します
//
void remove_char(char *buf,char chr) {

if (chr == ' ') {
      //コメントがある場合は削除
      for (int i=0 ; i<strlen(buf) ; i++) {
            if ((buf[i] == '/') && (buf[i+1] == '/')) {
                  buf[i]='\0';
                  break;
            }
      }
}


int len=strlen(buf);

for (int i=0 ; i<=len ; i++) {

      if (buf[i] == chr) {

            //対象文字を切り詰め削除する
            for (int j=i+1 ; j<=len ; j++) {
                  buf[j-1]=buf[j];
            }

            i--;
            len--;

       }
}

if (chr != ' ') { return; }

//半角スペース指定時はタブを消すために再帰
return remove_char(buf,'\t');


}



//  calc_splitstr( d0 , s0 , s1 )
//  分割予定の文字列の分割総数と分割後の最大文字数を取得
//  分割総数や最大文字数が不確定である場合に使うといいかも
//
//  引数: d0 文字数格納用int型変数ポインタ
//        s0 区切り文字列ポインタ
//        s1 分割予定文字列ポインタ
//
//  戻り値: -- 分割後の文字列総数
//          d0 分割後の最大文字数(実際の文字数＋ヌル文字)
//
//  区切り文字に任意の文字列を指定し実行させると
//  戻り値-1で文字列の出現回数を取得できる副産物検索機能
//
int calc_splitstr(int *wcnt,const char *token,const char *str) {

if (*str == 0) { *wcnt=0;
                 return 0; }

int col=0,cnt=0,wrd=0,wct=0,spt=0,flg=0;

for (int i=0 ; i<=strlen(str) ; i++) {

      wct++;

      for (cnt=0 ; cnt<strlen(token) ; cnt++) {

            //行末チェック
            if (*(str+col+cnt) == '\0') { col=strlen(str);
                                          flg=1;
                                          break; }

            //区切り文字でない場合はループを抜ける
            if (*(str+col+cnt) != token[cnt]) { col++;
                                                break; }

      }

      //トークンか行末時の処理
      if ((cnt == strlen(token)) || (flg))
          { spt++;
            col+=strlen(token);

            if (wrd < wct) { wrd=wct; }
            wct=0; }

      if (flg) { break; }

}

*wcnt=wrd;

return spt;


}



//  split_token( d0 , s0 , s1 , s2 )
//  文字列を指定区切り文字で分割し前方の文字列を返す
//  分割位置は自動で更新されるため呼び出した後で分割文字列を取得し
//  これを繰り返すことで文字列全てを分割することができます
//
//  引数:d0 文字列返し先ポインタ
//       s0 分割後の文字数制限値(期待される実際の文字数+1)(※)
//       s1 区切り文字格納元ポインタ
//       s2 分割前文字列格納元ポインタの「アドレス」
//  (※)10とした場合、９文字＋ヌル文字で返される
//
//  戻り値:  0 正常分割
//           1 終端により処理されず終了
//          -1 分割後の文字列が制限値を超過
//  上記に加えて、分割元文字列ポインタを区切り文字直後に移動
//  及び、区切り文字以前の文字列を返し先ポインタに対して格納する
//
//  注意：分割元文字列のポインタアドレスを「書き換えます」
//        分割元文字列自体を直接使うのではなく
//        確保後ポインタのコピーを作り使用してください
//
//        分割後、制限値を超えた文字列であった場合は
//        制限範囲分だけ返し先ポインタに格納されます
//        ２バイト以上の文字は半端な状態となる可能性があります
//
int split_token(char *buf,const int limit,const char *chr,char **str) {

if ((**str == '\0') || (limit < 1)) { return 1; }

int col=0,cnt=0;

for (cnt=0 ; cnt<strlen(chr) ; cnt++) {

      //行末チェック
      if (*(*str+col+cnt) == '\0') { col=strlen(*str);
                                     break; }

      //区切り文字でない場合はループ
      if (*(*str+col+cnt) != chr[cnt]) { cnt=-1;
                                         col++; }


}

//指定文字数超過の場合はエラー
//if (col >= limit) { return -1; }

int err=0;
//対象文字列を取得
for (int i=0 ; i<col ; i++) {

      if (i >= limit) { err=-1; break; }
      buf[i]=*(*str+i);

}

buf[((err) ? limit-1 : col)] = '\0';


//区切り文字で分割した場合は区切り文字分も桁を送る
if (cnt == strlen(chr)) { col+=strlen(chr); }

*str+=col;

return err;


}


//  get_config( d0 , s0 , s1 )
//  文字列から設定データを読み込む
//  再帰してます、作った人はあんまり再帰は好きじゃない
//
//  引数: d0 設定を保存するint型配列ポインタ「のアドレス」
//        s0 読み込む(再帰)回数
//        s1 設定文字列ポインタ「のアドレス」
//
//  戻り値 --  0 正常終了
//         -- -1 異常終了
//         d0 配列に設定用数値が代入されます
//
//  設定数値の異常チェックは別処理で行います
//
int get_config(int **cfg,int cnt,char **str) {

char buf[10];
if (split_token(buf,10,",",str)) { return -1; } //エラー
if (cc_strtol((*cfg)++,buf)) { return -1; } //エラー

if (--cnt == 0) { return 0; }

return get_config(cfg,cnt,str);


}



//  get_colorcode( d0 , s0 , s1 )
//  標準出力時のカラーコードを取得します
//
//  引数: d0 カラーコード取得用char型ポインタ(６バイト以上必須)
//        s0 ベースカラー番号
//        s1 カラーコード
//
//  戻り値: d0 -- カラーコード文字列を格納
//
//  d0で指定する文字列変数は必ず６バイト以上指定してください
//
void get_colorcode(char *buf,const int base,const int code) {

char str[]={"\033[00m"};	//５バイト＋ヌル文字

if (code == 0 || code == 7 || code > 9) { str[2]='3';
                                          str[3]='9'; }
                                   else {

      if (base) { str[2]='3'; }
           else { str[2]='9'; }

      str[3]=48+code;
}

strcpy(buf,str);


}



//  fileseek( s0 , s1 )
//  辞書ファイルの各カテゴリタグを頭出しします
//  正常終了の場合ファイルポインタはタグ直後の行に移動します
//
//  引数: s0 カテゴリタグ名の文字列ポインタ
//        s1 ファイルポインタ
//
//  戻り値:  0 正常終了
//          -1 異常終了(タグが見つからなかった)
//
int fileseek(const char *index,FILE *fp) {

char buf[COLUMN_SIZE]={0};
int  col=0,pos,err;

if (fseek(fp,0,SEEK_SET)) { show_error(1); }
errline=0;
while ((err=cc_fgets(buf,fp)) == 0) {

if (strcmp(buf,index) == 0) { return 0; }

}

return -1;


}



//  void convert_text( d0 , s0 , s1 , system_t )
//  本文中のタグを単語に入れ替えます
//
//  引数: d0 変換結果格納バッファ
//        s0 変換元char型文字列配列ポインタ
//        s1 辞書ファイルへのファイルポインタ
//
void convert_text(char *buf,const char *str,FILE *fp,const system_t *system) {

int write_pointer=0,err;
char word_buf[WORD_BUFFER];
char chr,cgr;

char def_color[6];
get_colorcode(def_color,system->base_color,system->color_code[0]);

for (int i=0 ; i<strlen(str) ; i++) {

      chr=str[i];
      cgr=0;
      if (chr == '%') {

            chr=str[i+1];
            if (chr == 'r') {

                  //乱数処理

                  chr=str[i+2];
                  if (chr != '[') { show_error(68); }

                  int pos=0;
                  for ( ; pos<12 ; pos++) {
                        chr=str[3+i+pos];
                        if (chr == ']') { break; }
                        if ((chr < '0') || (chr > '9')) { show_error(69); }
                        file_buffer[pos]=chr;
                  }

                  if (pos == 12) { show_error(69); }
                  file_buffer[pos]='\0';
                  i=i+(2+strlen(file_buffer));

                  if (cc_strtol(&pos,file_buffer)) { show_error(70); }

                  if (pos > 999999999) { show_error(71); }

                  pos=rand()%pos+1;
                  sprintf(file_buffer,"%d",pos);

                  cgr=7;

            }
      else if (chr == 'n') { cgr=1;
                             if (fileseek("[noun]",fp))         { show_error(18); } }
      else if (chr == 'p') { cgr=2;
                             if (fileseek("[place]",fp))        { show_error(19); } }
      else if (chr == 'a') { cgr=3;
                             if (fileseek("[adjective]",fp))    { show_error(20); } }
      else if (chr == 'j') { cgr=4;
                             if (fileseek("[na-adjective]",fp)) { show_error(21); } }
      else if (chr == 'w') { cgr=5;
                             if (fileseek("[weather]",fp))      { show_error(22); } }
      else if (chr == 'f') { cgr=6;
                             if (fileseek("[fortune]",fp))      { show_error(23); } }

      else if (chr == '\0') { buf[write_pointer++]='%';
                              i=strlen(str); }
      else { buf[write_pointer++]='%';
             buf[write_pointer++]=chr; }

                  if ((cgr >= 1) && (cgr <= 6)) {

                  //単語の取得
                  int rnd=rand()%system->counter[cgr]+1;

                  for (int j=0 ; j<rnd ; j++) {

                        if (cc_fgets(file_buffer,fp)) { show_error(29+cgr); }
                        if (file_buffer[0] == '\0')   { show_error(51+cgr); }

                        if (strlen(file_buffer) > WORD_LIMIT) { show_error(59+cgr); }

                  }
            }

            //カラーコード挿入
            get_colorcode(word_buf,system->base_color,system->color_code[cgr]);
            strcpy(&word_buf[5],file_buffer);
            strcpy(&word_buf[strlen(word_buf)],def_color);

            //単語を本文に挿入
            for (int j=0 ; j<strlen(word_buf) ; j++) {
                  buf[write_pointer++]=word_buf[j];
                  if (write_pointer >= BUFFER_SIZE) { show_error(5); } //エラー
            }

            i++;

      } else {

            //タグでない場合はパススルー
            buf[write_pointer++]=chr;
            if (write_pointer >= BUFFER_SIZE) { show_error(5); } //エラー

      }

}

buf[write_pointer]='\0';


}



//  void rebuild_dictionary( void )
//  辞書ファイルを再構成します
//  辞書ファイルを編集後、全カウンタ値を補正するためのものです
//
void rebuild_dictionary(void) {

FILE *ip;
if ((ip=fopen(DIC_FILEPATH,"r")) == NULL) { show_error(7); }


char category[7][15]={ {"[text]"        },{"[noun]"        }
                      ,{"[place]"       },{"[adjective]"   }
                      ,{"[na-adjective]"},{"[weather]"     }
                      ,{"[fortune]"     } };

int cct[7]={0};
int err;

for (int i=0 ; i<7 ; i++) {

      if (fileseek(category[i],ip)) { show_error(18+i); } //エラー

      while(1) {

            if ((err=cc_fgets(file_buffer,ip)) == -1) { show_error(28+i); } //エラー
            if ((file_buffer[0] == '\0') || (err == 1)) { break; }

            if (++cct[i] > CATEGORY_LIMIT) { show_error(38+i); } //エラー

      }

}


if (fseek(ip,0,SEEK_SET)) { show_error(1); }
char buf[COLUMN_LIMIT];

FILE *op;
if ((op=fopen(TMP_FILEPATH,"w")) == NULL) { show_error(3); }

int skip=0,pos;
while(fgets(file_buffer,COLUMN_SIZE,ip) != NULL) {

      if (strlen(file_buffer) > COLUMN_LIMIT) { show_error(6); } //必要ない気もする

      if (skip) { skip=0; continue; }

      strcpy(buf,file_buffer);
      pos=strlen(buf)-1;
      if (pos >= 0) { if (buf[pos] == 0x0a) { buf[pos--]='\0'; } }
      if (pos >= 0) { if (buf[pos] == 0x0d) { buf[pos--]='\0'; } }
      if (pos >= 0) { if (buf[pos] == 0x0a) { buf[pos--]='\0'; } }
      remove_char(buf,' ');

      if (strcmp(buf,"[counter]") == 0) {

            fputs(file_buffer,op);
            sprintf(file_buffer,"%d,%d,%d,%d,%d,%d,%d\n"
                   ,cct[0],cct[1],cct[2],cct[3],cct[4],cct[5],cct[6]);

            skip=1;

      }

      fputs(file_buffer,op);

}


fclose(op);
fclose(ip);

//remove(system->file_path);
rename(TMP_FILEPATH,DIC_FILEPATH);

printf("辞書ファイルの再構成に成功しました\n\n");

exit(0);


}



//  make_dictionary( void )
//  辞書ファイルを新規作成します
//  上書きチェックもあります
//
void make_dictionary(void) {

FILE *fp;
fp=fopen(DIC_FILEPATH,"r");

if (fp != NULL) {

      fclose(fp);

      printf("辞書ファイルは既に存在します\n");
      printf("辞書ファイルを作り直しますか？[y/N or ENTER]:");

      system("stty -icanon -echo");
      int chr,code;

      while(1) {

            chr=getchar();

            if ((chr >= 'a') && (chr <= 'z')) { chr-=32; }

            if (chr == 10 ) { chr='N'; }
            if (chr == 'N') { code=1; break; }
            if (chr == 'Y') { code=0; break; }

      }

      putchar(chr);
      printf("\n");

      system("stty icanon echo");

      if (code) { printf("中断しました\n\n");
                  exit(0); }

      printf("\n");

}


if ((fp=fopen(DIC_FILEPATH,"w")) == NULL) { show_error(2); }

int err;
err=fputs("//文字色指定(左から,ベースカラー,基本色,名詞[人名],名詞[地名],形容詞,形容動詞,天気,運勢,乱数)\n"
          "[color]\n"
          "0,0,1,2,3,5,6,5,5\n\n"
          "//辞書ファイル書式\n"
          "//[text]本文入力(%n/名詞[人名] %p/名詞[地名] %a/形容詞 %j/形容動詞 %w/天気 %r[x]/1-xまでの乱数\n"
          "//[noun]名詞(人名)\n"
          "//[place]名詞(地名)\n"
          "//[adjective]形容詞「美しい」の場合は「美し」と最後の「い」は削ること\n"
          "//[na-adjective]形容動詞\n"
          "//[weather]天気\n"
          "//[fortune]運勢\n"
          "[counter]\n5,5,5,5,5,5,5\n\n"
          "[text]\n"
          "本日%r[12]時頃、%p町で%nさん(%r[100])が%nのようなもので殴られる事件が発生、近隣住民によると被害者は「%jな性格で人に恨まれるとは思えない」と%aそうな表情で語っています。\n"
          "%n用洗剤の使用法：キャップ%r[10]杯分をスポンジにつけ汚れている%nを%aく拭き取る。%n洗剤と混ざると%jな%nを発生させる恐れがあります。誤って目に入ったり誤飲した場合は%aく%pに相談してください。\n"
          "%r[31]日%p大学にて講演を行っていた%nOS創始者%n氏がGPUベンダー%n社を非%j的であるとして中指を突き立て「%j！」と名指して非難した件で、同社CEO%n氏は%aく対応に追われているもようです。\n"
          "明日の天気：%p地方%w時々%w降水確率%r[100]％、%p地方%wのち%w降水確率%r[100]％、所により大量の%nが%aく降ってくる場合があります。\n"
          "占い師%nの今日の運勢：%n座のあなたは「%f」でラッキーポイント「%r[100]点」、%pで%aい出会いがあるかも？ラッキーアイテムは%n、これを手に%pを散歩すれば%jなことが起こるでしょう。\n\n"
          "[noun]\n"
          "ＭＳＸ\nリナックス\nロシア兵\n焼肉定食\nお好み焼き\n\n"
          "[place]\n"
          "東京\nニューヨーク\nシベリア\nラーメン屋\nエロマンガ島\n\n"
          "[adjective]\n"
          "嬉し\n悲し\n恐ろし\nバカバカし\n恥ずかし\n\n"
          "[na-adjective]\n"
          "有名\n悲劇\n静か\n顰蹙\n奇っ怪\n\n"
          "[weather]\n"
          "晴れ\n曇り\n雨\n雷雨\n雪\n\n"
          "[fortune]\n"
          "大吉\n中吉\n吉\n凶\n大凶\n",fp);

if (err == EOF) { show_error(2); }

fclose(fp);

printf("辞書ファイルを初期化しました\n\n");

exit(0);


}



//  show_ukagaka( system_t )
//  AAセレクタ(ターミナルでAA表示の実験場)
//  書いてのとおりでターミナル上でAAを表示したらどうなるか
//  の実験として入れたもの
//  気に入らなければ--hideオプションを使うなりどうなり
//
void show_ukagaka(system_t *system) {

int total=6;  //AA登録数、増減したら要変更
int rnd=0;

if (system->ukagaka == -1) { rnd=rand()%(total+1); }
                      else { rnd=system->ukagaka;  }

char bcl  = system->base_color;
char hide = system->hide_ukagaka;

char def_color[6];
get_colorcode(def_color,bcl,system->color_code[0]);

char  buf[256];

printf("- 人工無能");

//  AA追加時の注意点
//  人工無能～以降の表示はカラーコード含めて255文字までです
switch (rnd) {
case  0: get_colorcode(buf,bcl,6);
         strcpy(&buf[5],"Tux君");
         strcpy(&buf[strlen(buf)],def_color);
         printf("%sNews -\n",buf);
         if (hide == 0) { puts("      .--.\n     |o_o |\n     |:_/ |\n    //   \\ \\\n   (|     | )\n  /'\\_   _/`\\\n  \\___)=(___/"); }
         break;

case  1: get_colorcode(buf,bcl,2);
         strcpy(&buf[5],"ギコ猫のはにゃ～ん");
         strcpy(&buf[strlen(buf)],def_color);
         printf("%sNews -\n",buf);
         if (hide == 0) { puts("      ∧ ∧\n     ( ﾟДﾟ)\n     / つつ\n  ～(   ﾉ\n    し'Ｕ"); }
         break;

case  2: get_colorcode(buf,bcl,2);
         strcpy(&buf[5],"ながネギさん");
         strcpy(&buf[strlen(buf)],def_color);
         printf("%sNews -\n",buf);
         if (hide == 0) { puts("  　    /＾>》, -―‐‐＜＾}\n       /::::/,≠´:::::;::ヽ\n      /:::::〃::::／}::丿ハ\n     /:::::i{|::／　ﾉ／}::}\n    /::::::瓜イ ー ´ - ,':ﾉ\n   /:::::::|ﾉﾍ{､  (ﾌ_ノﾉイ\n  /::::::::| ／}｀不')::::"); }
         break;

case  3: get_colorcode(buf,bcl,6);
         strcpy(&buf[5],"(バカ)天才チルノ");
         strcpy(&buf[strlen(buf)],def_color);
         printf("%sNews -\n",buf);
         if (hide == 0) { puts("    __i＼_／!_\n    ゝ, \"´⌒｀ヽ\n     ﾉ.ノノﾉﾊﾉ〉\n  |＼ﾙﾘ! ﾟヮﾟﾉ!\n  ＼ k_(つ'i(つ\n  ∠ ,く／/ i ゝ\n     ｀!,ﾝｨﾝ\"´"); }
         break;

case  4: get_colorcode(buf,bcl,4);
         strcpy(&buf[5],"WindowsMe(嘘)");
         strcpy(&buf[strlen(buf)],def_color);
         printf("%sNews -\n",buf);
         if (hide == 0) { puts("        ⌒)\n   , '´￣￣｀ 、\n  /, , ,  ,.i､.ヽ\n  L|／x'_ノﾘxﾄ､|｣\n  ,ｱi´┃    ┃  |ﾇ\n  i !､\" ＿_ \",!i\n  ∨ ｎｰ｀-´＜ﾞ乂,\n   {｀i{f⌒iｦ　ﾞ爻、\n   ﾞｰ､∨ 、ﾉ ヽ `ｰ'\n       ＼/   ヽ"); }
         break;

case  5: get_colorcode(buf,bcl,1);
         strcpy(&buf[5],"アッガイ");
         strcpy(&buf[strlen(buf)],def_color);
         printf("%sNews -\n",buf);
         if (hide == 0) { puts("   ／：》ヽ\n  （===○ =）\n  （ づ旦c）\n   《 ＿)_）"); }
         break;

case  6: get_colorcode(buf,bcl,6);
         strcpy(&buf[5],"イカちゃん");
         strcpy(&buf[strlen(buf)],def_color);
         printf("%sNews -\n",buf);
         if (hide == 0) { puts("       ,.--  ―――――‐  、\n     ／      ,.-   ―― ｀ヽ\n　  /   |  ／:/: :ｨ:::::::＼\n   /　f⌒ h/':/l:/＼:/l/: :|｀\n  〈＿!l/ﾑl: :/  ＿  '＾jﾉ|\n    ＿| 人_|: l,,￣    =l:|.\n    {:_,::}｣: |   ｢ ７''}!:|_\n   ／　/=､|!: ト  l_/,ィ_└┐:|_\n  （  └勹ﾉi|: |＼､ //'ﾘ 〉＼;／"); }
         break;
}


}



//  void show_error( s0 )
//  エラーを表示します
//  エラー行表示のため暗黙的にグローバル変数errlineが使用されます
//
//  引数: s0 エラー番号
//
void show_error(int err) {

switch(err) {

case  1: printf(COLOR_RED"ERROR"COLOR_DEFAULT":ファイルシークに失敗しました\n");
         break;
case  2: printf(COLOR_RED"ERROR"COLOR_DEFAULT":辞書ファイルの作成に失敗しました\n");
         break;
case  3: printf(COLOR_RED"ERROR"COLOR_DEFAULT":一時ファイルの作成に失敗しました\n");
         break;

case  5: printf(COLOR_RED"ERROR"COLOR_DEFAULT":変換用バッファメモリが一杯になりました\n");
         break;
case  6: printf(COLOR_RED"ERROR"COLOR_DEFAULT":%d行の長さが制限値(%dバイト)を超えています\n",errline,COLUMN_LIMIT);
         break;
case  7: printf(COLOR_RED"ERROR"COLOR_DEFAULT":辞書ファイルが読み込めませんでした\n");
         break;

case 10: printf(COLOR_RED"ERROR"COLOR_DEFAULT":辞書ファイルが読み込めませんでした\n");
         printf("Tips :--makeオプションで辞書ファイルを作成できます\n");
         break;
case 11: printf(COLOR_RED"ERROR"COLOR_DEFAULT":辞書ファイルの%d行が制限値(%dバイト)を超えています\n",errline,COLUMN_LIMIT);
         break;

case 16: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[color]タグが見つかりませんでした\n");
         break;
case 17: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[counter]タグが見つかりませんでした\n");
         break;
case 18: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[text]タグが見つかりませんでした\n");
         break;
case 19: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[noun]タグが見つかりませんでした\n");
         break;
case 20: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[place]タグが見つかりませんでした\n");
         break;
case 21: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[adjective]タグが見つかりませんでした\n");
         break;
case 22: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[na-adjective]タグが見つかりませんでした\n");
         break;
case 23: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[weather]タグが見つかりませんでした\n");
         break;
case 24: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[fortune]タグが見つかりませんでした\n");
         break;

case 26: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[color]カラーコードの読み込みに失敗しました");
         break;
case 27: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[counter]カウンタの読み込みに失敗しました");
         break;
case 28: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[text]本文の読み込みに失敗しました");
         break;
case 29: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[noun]名詞(人名)の読み込みに失敗しました");
         break;
case 30: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[place]名詞(地名)の読み込みに失敗しました");
         break;
case 31: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[adjective]形容詞の読み込みに失敗しました");
         break;
case 32: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[na-adjective]形容動詞の読み込みに失敗しました");
         break;
case 33: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[weather]天気の読み込みに失敗しました");
         break;
case 34: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[fortune]運勢の読み込みに失敗しました");
         break;

case 36: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[counter]本文が制限値(%d)を超えています\n",CATEGORY_LIMIT);
         break;
case 37: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[counter]名詞(人名)が制限値(%d)を超えています\n",CATEGORY_LIMIT);
         break;
case 38: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[counter]名詞(地名)が制限値(%d)を超えています\n",CATEGORY_LIMIT);
         break;
case 39: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[counter]形容詞が制限値(%d)を超えています\n",CATEGORY_LIMIT);
         break;
case 40: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[counter]形容動詞が制限値(%d)を超えています\n",CATEGORY_LIMIT);
         break;
case 41: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[counter]天気が制限値(%d)を超えています\n",CATEGORY_LIMIT);
         break;
case 42: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[counter]運勢が制限値(%d)を超えています\n",CATEGORY_LIMIT);
         break;

case 45: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[color]不正なカラーコードが指定されています\n");
         break;
case 46: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[counter]不正なカウンタが指定されています\n");
         printf("Tips :--rebuildオプションで修復できるかもしれません\n");
         break;

case 50: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[text]本文の%d行目に何も入力されていない行があります\n",errline);
         break;
case 51: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[noun]名詞(人名)の%d行目に何も入力されていない行があります\n",errline);
         break;
case 52: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[place]名詞(地名)の%d行目に何も入力されていない行があります\n",errline);
         break;
case 53: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[adjective]形容詞の%d行目に何も入力されていない行があります\n",errline);
         break;
case 54: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[na-adjective]形容動詞の%d行目に何も入力されていない行があります\n",errline);
         break;
case 55: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[whether]天気の%d行目に何も入力されていない行があります\n",errline);
         break;
case 56: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[fortune]運勢の%d行目に何も入力されていない行があります\n",errline);
         break;

case 58: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[text]本文の%d行目の長さが制限値(%dバイト)を超えています\n",errline,TEXT_LIMIT);
         break;
case 59: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[noun]名詞(人名)の%d行目の長さが制限値(%dバイト)を超えています\n",errline,WORD_LIMIT);
         break;
case 60: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[place]名詞(地名)の%d行目の長さが制限値(%dバイト)を超えています\n",errline,WORD_LIMIT);
         break;
case 61: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[adjective]形容詞の%d行目の長さが制限値(%dバイト)を超えています\n",errline,WORD_LIMIT);
         break;
case 62: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[na-adjective]形容動詞の%d行目の長さが制限値(%dバイト)を超えています\n",errline,WORD_LIMIT);
         break;
case 63: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[whether]天気の%d行目の長さが制限値(%dバイト)を超えています\n",errline,WORD_LIMIT);
         break;
case 64: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[fortune]運勢の%d行目の長さが制限値(%dバイト)を超えています\n",errline,WORD_LIMIT);
         break;

case 68: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[text]本文%d行目の乱数タグに数値が指定されていません\n",errline);
         break;
case 69: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[text]本文%d行目の乱数タグの数値が括弧で閉じられていません\n",errline);
         break;
case 70: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[text]本文%d行目の乱数タグの数値の変換に失敗しました\n",errline);
         break;
case 71: printf(COLOR_RED"ERROR"COLOR_DEFAULT":[text]本文%d行目の乱数タグの数値が最大値(999999999)を超えています\n",errline);
         break;

}

printf("\n");

exit(-1);


}


//  void help( s0 )
//  ヘルプを表示します
//
//  引数: s0 隠しヘルプ表示フラグ
//
void help(char kakushi) {

puts("人工無能Tux君ニュース C言語版 Version "VERSION_INFO"\n"
      COPYRIGHT"\n\n"

     "使用法:"SOFTWARE_FILE"... ニュースを表示\n"
     "または:"SOFTWARE_FILE" --make ....... 辞書ファイル作成\n"
     "または:"SOFTWARE_FILE" --rebuild .... 辞書ファイル再構成\n"
     "または:"SOFTWARE_FILE" --hide ....... AA非表示\n"
     "または:"SOFTWARE_FILE" --page n ..... 任意の本文に固定\n"
     "または:"SOFTWARE_FILE" --help / -h .. ヘルプ表示\n");


if (kakushi) {

puts("または:"SOFTWARE_FILE" --kakushi .... 隠しヘルプ\n"
     "または:"SOFTWARE_FILE" --giko ....... ギコ猫のはにゃ～んニュース\n"
     "または:"SOFTWARE_FILE" --negi ....... ながネギさんニュース\n"
     "または:"SOFTWARE_FILE" --baka ....... チルノの(９)ニュース\n"
     "または:"SOFTWARE_FILE" --blue ....... WindowsMe(嘘)ニュース\n"
     "または:"SOFTWARE_FILE" --zaku ....... アッガイニュース\n"
     "または:"SOFTWARE_FILE" --ika ........ イカちゃんニュース\n"
     "または:"SOFTWARE_FILE" --nini ....... AAをランダム選択\n");

}

exit(0);


}



//  void set_option( s0 , d0 , system_t )
//  コマンドオプションを調べ設定します
//
//  引数: s0 argcをそのまま送る
//        s1 argvをそのまま送る
//
//  結果はsystem_t構造体の変数に格納されます
//
void set_option(int argc,char **argv,system_t *system) {

if (argc < 2) { return; }

for (int i=1 ; i<argc ; i++) {

      if (strcmp(argv[i],"--help") == 0) { system->help=1; }
            else if (strcmp(argv[i],"-h"       ) == 0) { system->help=1; }
            else if (strcmp(argv[i],"--make"   ) == 0) { system->make_dic=1; }
            else if (strcmp(argv[i],"--rebuild") == 0) { system->rebuild_dic=1; }
            else if (strcmp(argv[i],"--hide"   ) == 0) { system->hide_ukagaka=1; }

            else if (strcmp(argv[i],"--kakushi") == 0) { system->kakushi=1; }

            else if (strcmp(argv[i],"--giko") == 0) { system->ukagaka= 1; }
            else if (strcmp(argv[i],"--negi") == 0) { system->ukagaka= 2; }
            else if (strcmp(argv[i],"--baka") == 0) { system->ukagaka= 3; }
            else if (strcmp(argv[i],"--blue") == 0) { system->ukagaka= 4; }
            else if (strcmp(argv[i],"--zaku") == 0) { system->ukagaka= 5; }
            else if (strcmp(argv[i],"--ika" ) == 0) { system->ukagaka= 6; }
            else if (strcmp(argv[i],"--nini") == 0) { system->ukagaka=-1; }

            else if (strcmp(argv[i],"--page") == 0) {

            i++;
            if (i < argc) {
                  int num;
                  if (cc_strtol(&num,argv[i]) == 0) {
                        system->page_number=num;
                  }

            }

      }

}


}


