#! /usr/bin/perl
use Encode;
use strict;
use utf8;
use warnings;

#
#  Linux News for Perl / ver0.000-1α(17/03/31)
#  Copyright (C)2017 etokami
#

#注意:Windows、MacOSでの動作は保証なし

#ファイルパスとファイルネーム設定
#ファイルパスは末尾にスラッシュ必須
our $logfile_filepath = "./";
our $logfile_filename = "news.dic";
our $logfile_tempfile = "temp.log";

#無限ループ抑止用カウンタ上限値(変更非推奨)
#言い換えるとタグの変換可能最大数
#よって小さすぎると変換しきれない場合がある
#大きすぎても正常に動作している場合影響はない
our $break_count  = 256;

#色々設定用(変更禁止)
our $version_info = "0.000-1α";
our $color_base   = 0;    #色変数ベース配列番号
our $color_text   = 1;    #色設定本文用配列番号
our $color_tag    = 2;    #色設定各タグ用配列番号
our $color_rnd    = 8;    #色設定乱数用配列番号
our $ukagaka=0;           #AAセレクタ
our $hide=0;             #AA表示フラグ
our $page=0;             #本文セレクタ
our $message="";         #本文用バッファ
our @section = ("[text]","[noun]","[place]","[adjective]"
              ,"[na-adjective]","[weather]","[fortune]");
our @tag     = ('%n','%p','%a','%j','%w','%f');
our @color   = ();
our @counter = ();


#
#  ここからメイン
#

binmode(STDOUT,"utf8");

#コマンド引数処理
my @argv=@ARGV;

my ($i);
for ($i=0 ; $i<=$#argv ; $i++) {
   if ($argv[$i] eq "--make")    { &make;
                                   exit 0; }
   if ($argv[$i] eq "--rebuild") { &rebuild;
                                   exit 0; }
   if ($argv[$i] eq "--hide")    { $hide=1; }
   if ($argv[$i] eq "--help" || $argv[$i] eq "-h") { &help(0);
                                                     exit 0; }

   if ($argv[$i] eq "--kakushi") { &help(1);
                                   exit 0; }
   if ($argv[$i] eq "--giko") { $ukagaka=1; }
   if ($argv[$i] eq "--negi") { $ukagaka=2; }
   if ($argv[$i] eq "--baka") { $ukagaka=3; }
   if ($argv[$i] eq "--blue") { $ukagaka=4; }
   if ($argv[$i] eq "--zaku") { $ukagaka=5; }
   if ($argv[$i] eq "--ika" ) { $ukagaka=6; }
   if ($argv[$i] eq "--nini") { $ukagaka=-1; }
   if ($argv[$i] eq "--page") {
   if ($i < $#argv) { $page=$argv[$i+1]; }
      }
   }

&main;

exit 0;


#以下サブルーチン

#
#  改行、スペース、タブ、コメントを削除
#
sub remove_space {

my ($a,$b,$i,$v);

$v=$_[0];
$v=~s/\n|\r|\r\n//g;
$v=index($v,'//');
$a=substr($_[0],0,$v);

for ($i=length($a) ; $i>=0 ; $i--) {
   $b=substr($a,$i-1,1);
   if ($b ne " " && $b ne "\t") { last; }
   }

return substr($a,0,$i);

}


#
#  カラースキーム取得
#
sub get_color {

my $a=3;
if ($color[$color_base] == 0) {
   $a=($color[$_[0]]) ? 9 : 3;
   }

my $b=$_[0];
if ($b == 0 || $b == 7 || $b > 9) { $b="9"; }

return "\33["."$a$b"."m";

}


#
#  セクション直後にシーク
#
sub search_section {

my ($a);

seek(IN,0,0);
while (<IN>) {
   $a=&remove_space($_);
   if ($a eq $_[0]) { return 0; }
   }

return 1;

}


#
#  ランダムに１行取得
#
sub get_line {

my ($a,$i);

my $rnd=0;
if ($_[0] > 0) { $rnd=int(rand($_[0])); }
    elsif ($_[0] < 0) { $rnd=abs($_[0])-1; }

for ($i=0 ; $i<=$rnd ; $i++) {
   $a=<IN>;
   $a=&remove_space($a);
   if ($a=~/^\[/) { return ""; }
   if ($a eq "") { $i--; }
   }

return $a;

}


#
#  乱数以外のタグを置き換え
#
sub replace_tag {

my ($a,$i);

if (&search_section($section[$_[0]+1])) { &error(21+$_[0]); }

$a=&get_line($counter[$_[0]+1]);
if ($a eq "") { &error(41+$_[0]); }
$a=&get_color($color[$_[0]+$color_tag]).$a.&get_color($color[$color_text]);

$message=~s/$tag[$_[0]]/$a/;

}


sub ukagaka {

my ($a,$b,$c);
my $doll_entry=7;

#AAをランダム選出
if ($ukagaka == -1) { $ukagaka=int(rand($doll_entry)); }

if ($ukagaka == 0) {       $a="      .--.\n     |o_o |\n     |:_/ |\n    //   \\ \\\n   (|     | )\n  /'\\_   _/`\\\n  \\___)=(___/";
                          $b="Tux君";
                          $c=6; }
   elsif ($ukagaka == 1) { $a="      ∧ ∧\n     ( ﾟДﾟ)\n     / つつ\n  ～(   ﾉ\n    し'Ｕ";
                          $b="ギコ猫のはにゃ～ん";
                          $c=2; }
   elsif ($ukagaka == 2) { $a="  　    /＾>》, -―‐‐＜＾}\n       /::::/,≠´:::::;::ヽ\n      /:::::〃::::／}::丿ハ\n     /:::::i{|::／　ﾉ／}::}\n    /::::::瓜イ ー ´ - ,':ﾉ\n   /:::::::|ﾉﾍ{､  (ﾌ_ノﾉイ\n  /::::::::| ／}｀不')::::";
                          $b="ながネギさん";
                          $c=2; }
   elsif ($ukagaka == 3) { $a="    __i＼_／!_\n    ゝ, \"´⌒｀ヽ\n     ﾉ.ノノﾉﾊﾉ〉\n  |＼ﾙﾘ! ﾟヮﾟﾉ!\n  ＼ k_(つ'i(つ\n  ∠ ,く／/ i ゝ\n     ｀!,ﾝｨﾝ\"´";
                          $b="(バカ)天才チルノ";
                          $c=6; }
   elsif ($ukagaka == 4) { $a="        ⌒)\n   , '´￣￣｀ 、\n  /, , ,  ,.i､.ヽ\n  L|／x'_ノﾘxﾄ､|｣\n  ,ｱi´┃    ┃  |ﾇ\n  i !､\" ＿_ \",!i\n  ∨ ｎｰ｀-´＜ﾞ乂,\n   {｀i{f⌒iｦ　ﾞ爻、\n   ﾞｰ､∨ 、ﾉ ヽ `ｰ'\n       ＼/   ヽ";
                          $b="WindowsMe(嘘)";
                          $c=4; }
   elsif ($ukagaka == 5) { $a="   ／：》ヽ\n  （===○ =）\n  （ づ旦c）\n   《 ＿)_）";
                          $b="アッガイ";
                          $c=1; }
   elsif ($ukagaka == 6) { $a="       ,.--  ―――――‐  、\n     ／      ,.-   ―― ｀ヽ\n　  /   |  ／:/: :ｨ:::::::＼\n   /　f⌒ h/':/l:/＼:/l/: :|｀\n  〈＿!l/ﾑl: :/  ＿  '＾jﾉ|\n    ＿| 人_|: l,,￣    =l:|.\n    {:_,::}｣: |   ｢ ７''}!:|_\n   ／　/=､|!: ト  l_/,ィ_└┐:|_\n  （  └勹ﾉi|: |＼､ //'ﾘ 〉＼;／";
                          $b="イカちゃん";
                          $c=6; }
print "- 人工無能".&get_color($c).$b.&get_color($color[$color_text])."News -\n";

if ($hide == 0) { print "$a\n"; }

}


#
#  辞書ファイル作成
#
sub make {

binmode(STDIN,"utf8");

if (-f "$logfile_filepath$logfile_filename") {
   print "既存の辞書ファイルを破棄して新規に作成します(y/N)";
   $_=<STDIN>;
   if (! /y/i) { print "中断しました\n\n";
                 return; }
   }
open(OUT,">$logfile_filepath$logfile_filename") || &error(4);
binmode(OUT,'utf8');

print OUT <<END_OF_DATA;
//文字色指定(左から,ベースカラー,基本色,名詞[人名],名詞[地名],形容詞,形容動詞,天気,運勢,乱数)
[color]\n0,0,1,2,3,5,6,5,5\n
//辞書ファイル書式
//[text]本文入力(%n/名詞[人名] %p/名詞[地名] %a/形容詞 %j/形容動詞 %w/天気 %r[x]/1-xまでの乱数
//[noun]名詞(人名)
//[place]名詞(地名)
//[adjective]形容詞「美しい」の場合は「美し」と最後の「い」は削ること
//[na-adjective]形容動詞
//[weather]天気
//[fortune]運勢
[counter]\n5,5,5,5,5,5,5\n
[text]
本日%r[12]時頃、%p町で%nさん(%r[100])が%nのようなもので殴られる事件が発生、近隣住民によると被害者は「%jな性格で人に恨まれるとは思えない」と%aそうな表情で語っています。
%n用洗剤の使用法：キャップ%r[10]杯分をスポンジにつけ汚れている%nを%aく拭き取る。%n洗剤と混ざると%jな%nを発生させる恐れがあります。誤って目に入ったり誤飲した場合は%aく%pに相談してください。
%r[31]日%p大学にて講演を行っていた%nOS創始者%n氏がGPUベンダー%n社を非%j的であるとして中指を突き立て「%j！」と名指して非難した件で、同社CEO%n氏は%aく対応に追われているもようです。
明日の天気：%p地方%w時々%w降水確率%r[100]％、%p地方%wのち%w降水確率%r[100]％、所により大量の%nが%aく降ってくる場合があります。
占い師%nの今日の運勢：%n座のあなたは「%f」でラッキーポイント「%r[100]点」、%pで%aい出会いがあるかも？ラッキーアイテムは%n、これを手に%pを散歩すれば%jなことが起こるでしょう。\n
[noun]
ＭＳＸ\nリナックス\nロシア兵\n焼肉定食\nお好み焼き\n
[place]
東京\nニューヨーク\nシベリア\nラーメン屋\nエロマンガ島\n
[adjective]
嬉し\n悲し\n恐ろし\nバカバカし\n恥ずかし\n
[na-adjective]
有名\n悲劇\n静か\n顰蹙\n奇っ怪\n
[weather]
晴れ\n曇り\n雨\n雷雨\n雪\n
[fortune]
大吉\n中吉\n吉\n凶\n大凶
END_OF_DATA

close(OUT);
$_=chmod 0644,"$logfile_filepath$logfile_filename";
if ($_ != 1) { &error(7); }

print "辞書ファイルを作成しました\n\n";

}


#
#  辞書ファイルの再構成
#
sub rebuild {

my ($a,$i);
my @cnt=();

open(IN,"$logfile_filepath$logfile_filename") || &error(0);
binmode(IN,"utf8");

for ($i=0 ; $i<=$#section ; $i++) {
   if (&search_section($section[$i])) { &error(20+$i); }
   $cnt[$i]=0;

   while ($a=<IN>) {
      $a=&remove_space($a);
      if ($a=~/^\[/) { last; }
      if ($a ne "") { $cnt[$i]++; }
      }
   }

seek(IN,0,0) || &error(0);

open(OUT,">$logfile_filepath$logfile_tempfile") || &error(5);
binmode(OUT,"utf8");

while ($_=<IN>) {
$a=&remove_space($_);
if ($a eq "[counter]") { print OUT $_;
                         <IN>;
                         $_="$cnt[0]";
                         for ($i=1 ; $i<=$#cnt ; $i++) { $_.=",$cnt[$i]"; }
                         $_.="\n";
                       }
print OUT $_;
}

close(IN);
close(OUT);

rename("$logfile_filepath$logfile_tempfile","$logfile_filepath$logfile_filename") || &error(6);
$_=chmod 0644,"$logfile_filepath$logfile_filename";
if ($_ != 1) { &error(8); }

print "辞書ファイルを再構成しました\n\n";

}


#
#  ヘルプ
#
sub help {

print <<END_OF_DATA;
Linux News for Perl / Version $version_info
使用法:linux_news.pl ... ニュースを表示
または:linux_news.pl --make ....... 辞書ファイル作成
または:linux_news.pl --rebuild .... 辞書ファイル再構成
または:linux_news.pl --hide ....... AA非表示
または:linux_news.pl --page n ..... 任意の本文に固定
または:linux_news.pl --help / -h .. ヘルプ表示\n
END_OF_DATA

if ($_[0] == 0) { return; }

print <<END_OF_DATA;
または:linux_news.pl --kakushi .... 隠しヘルプ
または:linux_news.pl --giko ....... ギコ猫のはにゃ～んニュース
または:linux_news.pl --negi ....... ながネギさんニュース
または:linux_news.pl --baka ....... チルノの(９)ニュース
または:linux_news.pl --blue ....... WindowsMe(嘘)ニュース
または:linux_news.pl --zaku ....... アッガイニュース
または:linux_news.pl --ika ........ イカちゃんニュース
または:linux_news.pl --nini ....... AAをランダム選択\n
END_OF_DATA

}


#
#  エラー処理
#
sub error {
my ($a,$b);
$b="";

if ($_[0] == 0) {        $a="辞書ファイルが見つかりません";
                         $b="--makeオプションで辞書ファイルを作成できます"; }
  elsif ($_[0] ==   1) { $a="[color] / セクションが見つかりません"; }
  elsif ($_[0] ==   2) { $a="[counter] / セクションが見つかりません"; }
  elsif ($_[0] ==   3) { $a="[counter] / カウンタ値が不正です";
                         $b="--rebuildオプションで修復できる可能性があります"; }
  elsif ($_[0] ==   4) { $a="--make / 辞書ファイルの作成に失敗しました"; }
  elsif ($_[0] ==   5) { $a="--rebuild / 一時ファイルの作成に失敗しました"; }
  elsif ($_[0] ==   6) { $a="--rebuild / 一時ファイルのリネームに失敗しました";
                         $b=$logfile_tempfile."を".$logfile_filename."に手動でリネームすると修復できる可能性があります"; }
  elsif ($_[0] ==   7) { $a="--make / パーミッションの設定に失敗しました"; }
  elsif ($_[0] ==   8) { $a="--rebuild / パーミッションの設定に失敗しました"; }
  elsif ($_[0] ==  10) { $a="乱数の桁数が多すぎる箇所があります"; }
  elsif ($_[0] ==  20) { $a="[text] / セクションが見つかりません"; }
  elsif ($_[0] ==  21) { $a="[noun] / セクションが見つかりません"; }
  elsif ($_[0] ==  22) { $a="[place] / セクションが見つかりません"; }
  elsif ($_[0] ==  23) { $a="[adjection] / セクションが見つかりません"; }
  elsif ($_[0] ==  24) { $a="[na-adjection] / セクションが見つかりません"; }
  elsif ($_[0] ==  25) { $a="[weather] / セクションが見つかりません"; }
  elsif ($_[0] ==  26) { $a="[fortune] / セクションが見つかりません"; }
  elsif ($_[0] ==  40) { $a="[text] / カウンタ値が不正です";
                         $b="--rebuildオプションで修復できる可能性があります"; }
  elsif ($_[0] ==  41) { $a="[noun] / カウンタ値が不正です";
                         $b="--rebuildオプションで修復できる可能性があります"; }
  elsif ($_[0] ==  42) { $a="[place] / カウンタ値が不正です";
                         $b="--rebuildオプションで修復できる可能性があります"; }
  elsif ($_[0] ==  43) { $a="[adjection] / カウンタ値が不正です";
                         $b="--rebuildオプションで修復できる可能性があります"; }
  elsif ($_[0] ==  44) { $a="[na-adjection] / カウンタ値が不正です";
                         $b="--rebuildオプションで修復できる可能性があります"; }
  elsif ($_[0] ==  45) { $a="[weather] / カウンタ値が不正です";
                         $b="--rebuildオプションで修復できる可能性があります"; }
  elsif ($_[0] ==  46) { $a="[fortune] / カウンタ値が不正です";
                         $b="--rebuildオプションで修復できる可能性があります"; }
  elsif ($_[0] == 255) { $a="ループカウンタトラップにかかりました"; }

print "Error: $a\n";
if ($b) { print "Tips : $b\n"; }
print "\n";

exit 1;

}


#
#  メイン処理
#
sub main {

my ($rnd,$a,$i);

#乱数シード初期化
srand;

#辞書ファイルオープン
open(IN,"$logfile_filepath$logfile_filename") || &error(0);
binmode(IN,"utf8");

#カラースキーム取得
if (&search_section("[color]")) { &error(1); }
@color=split(",",&get_line(0));

#カウンタ取得
if (&search_section("[counter]")) { &error(2); }
@counter=split(",",&get_line(0));
foreach (@counter) {
   if ($_ < 1) { &error(3); }
   }

#本文抽出
if (&search_section("[text]")) { &error(20); }
$a=($page) ? $page*-1 : $counter[0];
$message=&get_line($a);
if ($message eq "") { &error(40); }

my ($cnt,$flag);
$cnt=0;
do {
$flag=0;

#乱数以外を置換
for ($i=0 ; $i<=$#tag ; $i++) {
   if ($message=~/$tag[$i]/) { &replace_tag($i);
                               $flag++; }
   }

#乱数を置換
if ($message=~/%r\[([0-9]+)\]/) {
   if (length($1) > 9) { &error(10); }
   $rnd=int(rand($1))+1;
   $rnd=&get_color($color[$color_rnd]).$rnd.&get_color($color[$color_text]);
   $a=$&;
   $a=~s/\[/\\\[/;
   $a=~s/\]/\\\]/;
   $message=~s/$a/$rnd/;
   $flag++;
   }

#保険
if (++$cnt > $break_count) { &error(255); }

} while ($flag);

close(IN);

&ukagaka;

print "$message\n";
print "\n";

}
